#!/usr/bin/perl -w

# LSB Test Execution Framework
# Help Page Module (tests_help.pl)
#
# Copyright (C) 2006-2007 Free Standards Group. All rights reserved.
#
# This program has been developed by ISP RAS for FSG.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n\n";

print_header("Information", "help");

print <<DATA;
<a name="top"></a><h1>Information</h1>
<p align="justify"><b>LSB Distribution Testkit</b> provides the web-oriented interface for executing LSB tests. The web-interface contains the following pages:
</p>
<ol>
  <li><a href="#cert">Get&nbsp;Certified</a></li>
  <li><a href="#conf">Configure</a></li>
  <li><a href="#exec">Execution</a></li>
  <li><a href="#results">Results</a></li>
  <li><a href="#help">Help</a></li>
  <li><a href="#about">About</a></li>
  <li><a href="#admin">Administration</a></li>
</ol>
<br />
<a name="cert"></a><h2>Get&nbsp;Certified</h2>
<p align="justify">
Open this page if your aim is to get LSB-Compliance certification. This page is very similar to the <a href="#conf"><b>Configure</b></a> but has some restrictions implied by the certification process:
</p>
<ul>
<li>The <b>Your name</b> and <b>Organization</b> fields are mandatory.</li>
<li>It is impossible to turn off using Internet: all the test packages must be downloaded by the LSB Distribution Testkit if they are not available on the local hard drives.</li>
<li>It is impossible to use short scenarios.</li>
<li>Tests selection list is turned off (all the necessary tests are executed).</li>
</ul>
<p align="justify">
After the tests execution is finished the browser is automatically redirected to the special results page, which represents a simplified variant of the tests execution report. Also this page describes the further steps on getting the certification.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="conf"></a><h2>Configure</h2>
<p align="justify">
This page allows you to configure the test system. The following options are present:
</p>
<ul>
<li><b>Your name:</b><br />The name of the user who started the tests execution. This field is reserved for future use (currently not used).</li>
<li><b>Organization:</b><br />The name of your organization. This field is reserved for future use (currently not used).</li>
<li><b>E-mail:</b><br />Specify your E-mail here. It will be used for sending the report after the tests finish.</li>
<li><b>Send report via E-mail</b><br />This checkbox allows to turn off sending the report without removing the E-mail address.</li>
<li><b>Select the architecture:</b><br />This combobox allows you to select the machine architecture. Please note that only those architectures are listed that are compatible with the machine being tested.</li>
<li><b>Use Internet connection for downloading the data</b><br />This checkbox allows you to specify not to use any Internet connections for downloading the test packages.</li>
<li><b>Refresh list</b><br />This button makes the Framework to connect to the <a href="ftp://freestandards.org/" target="_blank">FSG's FTP server</a> and obtain the list of the latest test packages. Then the list of packages show below is updated, according to this information and to the list of the already downloaded packages.</li>
<li><b>Additional details</b><br />A group of the additional settings. To show/hide them just click this orange header. This block contains the following settins:
  <ul>
  <li><b>Keep packages</b><br />This option allows to keep packages that were installed for tests execution. If this option is turned off, the packages are deinstalled after the tests are finished. Else the packages are kept installed on the system (this will allow to start tests next time without reinstalling the packages which saves time).<br /><font color="blue"><b>Warning:</b></font> If you set the <b>Keep packages</b> option, please, avoid mixing running tests for different architectures. The Framework cannot determine for which architecture the installed package is designed, so there a situation may occur when you are trying to run some test for <b>amd64</b> platform, and the same test for <b>ia32</b> platform is started instead&nbsp;&ndash; just because this test package had already been installed before, and reused now.</li>
  <li><b>Short scenarios</b><br />This option allows to run only reduced set of tests, which will result in much shorter time of executing. It is present only in test versions of the LSB Distribution Testkit and designed to simplify the process of reproducing errors when providing feedback.</li>
  <li><b>Verbose Level:</b><br />Specifies how much information should be given by the tests into the output console.</li>
  <li><b>LSB Version:</b><br />This combobox allows to select the LSB version to check for compliance with.</li>
  <li><b>Comments:</b><br />Here you may write comments with any additional information. These comments are stored in the execution report and shown in the <a href="#results"><b>Results</b></a> page.</li>
  </ul>
</li>
</ul>
<p align="justify">
Below is the list of available test packages present. At first start it will most likely be empty. To get the full list press the <b>Refresh list</b> button (if the list is empty this button changes its name to <b>Get List</b>). It may take about 20 to 40 seconds depending on your Internet connection speed and load of the servers. The list from FTP is cached, so you don't need to refresh it every time. If you did not refresh the list for too long (more than 24 hours), the LSB Distribution Testkit shows you a warning that the list may be out-of-date.<br />
The list of test suites consists of the four columns. The first column contains the name of the test suite and a checkbox indicating whether to execute this test or not (if it was set and then disabled, e.g. when you turned off the <b>Use Internet connection</b> checkbox, the test will not be executed). The three last columns allow to choose which version of the test suite to execute. These columns are:
</p>
<ol>
<li><b>Localhost</b><br />The version of the test suite that is present on your local hard disk (either downloaded into <font color="blue">/var/opt/lsb/distr-testkit/lsb-packages</font> directory or one of its subdirectories, or already installed).</li>
<li><b>FTP</b><br />The latest <b>stable</b> version present on the FSG's FTP server.</li>
<li><b>FTP (beta)</b><br />The latest <b>beta</b> version present on the FSG's FTP server.</li>
</ol>
<p align="justify">
If some of the items do not contain any version number, it means that the corresponding version is not available.
</p>
<p align="justify">
When you select the tests, you can see the approximate summary size of the packages to be downloaded (at the top of the page, the <b>Download size:</b> field). If you turn the option <b>Use Internet connection for downloading the data</b> off, all test packages which need downloading will be automatically disabled.
</p>
<p align="justify">
After you selected the test packages to run you may press the button <b>Run selected tests</b> located at the bottom of the page. This will redirect you immediately to the next page (<a href="#exec"><b>Execution</b></a>) and the tests will start. If the tests are already (or still) running you will get the error message at the top of the page, because it is currently impossible to run several tests instances simultaneously.
</p>
<h3>Profile Management</h3>
<p align="justify">
This section allows you to save/load all the settings listed above in different profile files. Just specify the profile name in the <b>Save profile:</b> field and press the <b>Save</b> button. To load a previously saved profile, select it in the combobox <b>Load profile:</b> and press <b>Load</b>. The button <b>Delete</b> allows you to delete the old profiles that are no longer needed.<br />
<b>Note:</b> The profile is not automatically saved when you run the tests! The settings are just stored in a temporary session profile which is automatically removed when it is no longer used. So if you wish to use the same set of options later, please save it explicitly, via the <b>Save</b> button.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="exec"></a><h2>Execution</h2>
<p align="justify">
This page allows you to watch the tests execution progress. If the tests are already running, you may open this page to watch the progress (even from another computer). If the tests are not running at the moment, you may select the profile that was saved earlier on the <a href="#conf"><b>Configure</b></a> page and run the tests by pressing the <b>Start</b> button. The <b>Stop</b> button allows you to forcibly stop the tests execution.
</p>
<p align="justify">
Just below the current status of the tests execution and its output console are shown. The console is refreshed automatically with the period of about 1 second and displays all the tests output (both <b>stdout</b> and <b>stderr</b>). If the tests have just been finished, the status may contain the error code returned by them. If the tests were stopped, the received signal is shown.
</p>
<p align="justify">
If the report has been created, the link <b>View test report</b> is shown below the output console. Just click it to view this report.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="results"></a><h2>Results</h2>
<p align="justify">
This page shows the table with the full list of the tests execution results. The table contains the following columns:
</p>
<ul>
<li><b>Date/Time</b><br />Here the date and time are shown, when the tests were finished. At the left edge of this column a status-icon is shown:
<ul>
<li><img src="images/CertPassed.png" alt="CertPass" width="33" height="30" /> The certification tests passed.</li>
<li><img src="images/CertFailed.png" alt="CertFail" width="33" height="30" /> The certification tests failed.</li>
<li><img src="images/TestPassed.png" alt="Pass" width="33" height="30" /> The custom set of tests passed.</li>
<li><img src="images/TestFailed.png" alt="Fail" width="33" height="30" /> The custom set of tests failed.</li>
</ul>
</li>
<li><b>Summary</b><br />This column shows the general results of the tests (how many tests passed and failed).</li>
<li><b>Comment</b><br />Here the comments entered on the <a href="#conf"><b>Configure</b></a> page are shown.</li>
</ul>
<p align="justify">
To view the detailed particular report just click the link in the <b>Date/Time</b> field.
</p>
<p align="justify">
To remove the no longer needed reports, just mark them with the checkboxes shown at the left edge of the corresponding lines and press the <b>Remove selected</b> button below the table.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="help"></a><h2>Help</h2>
<p align="justify">
Contains the description of the web-interface of LSB Distribution Testkit (the page you are reading now).
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="about"></a><h2>About</h2>
<p align="justify">
Contains the general information about LSB Distribution Testkit: its version, license etc.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="admin"></a><h2>Administration</h2>
<p align="justify">
This page is designed for performing administrative tasks. Please, be careful with any actions on this page!
</p>
<p align="justify">
Currently the following items are present:
</p>
<ul>
<li><b>Server status:</b><br />Shows current status of the web-server and its PID. The button <b>Stop server</b> allows to stop its execution. Please note that you will not be able to use web-interface after that!</li>
<li><b>Tests status:</b><br />Shows current status of the tests and PID of their parent process. The button <b>Stop tests</b> allows to stop the tests execution (it has the same meaning as the <b>Stop</b> button on the <a href="#exec"><b>Execution</b></a> page).</li>
</ul>
<p align="right"><a href="#top">Back to top</a></p>
DATA

print_footer();
