#!/usr/bin/perl -w

# LSB Test Execution Framework
# Web-server Module (server.pl)
#
# Copyright (C) 2006-2007 Free Standards Group. All rights reserved.
#
# This program has been developed by ISP RAS for FSG.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use strict;
use Socket;
use IO::Socket;
use POSIX ":sys_wait_h";
use Fcntl ":flock";

# $DOCUMENT_ROOT - path to directory connected to http://localhost:8888/
# It's the 'public_html' subdirectory in the server.pl script directory.
my $SERVER_ROOT = `pwd`;
$SERVER_ROOT =~ s/[\r\n]//g;
$SERVER_ROOT .= "/".$0;
$SERVER_ROOT =~ s!/[^/]+$!!;
chdir($SERVER_ROOT);
$SERVER_ROOT = `pwd`;
$SERVER_ROOT =~ s/[\r\n]//g;
my $DOCUMENT_ROOT = $SERVER_ROOT."/public_html";

my @index_docs = ("index.html", "index.htm", "index.pl", "index.cgi");
my %mime_types;

sub LOG($) {
	flock(STDERR, LOCK_EX);
	my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
	my $date = sprintf("%02d.%02d.%04d %02d:%02d:%02d", $mday, $mon, $year + 1900, $hour, $min, $sec);
	print STDERR "$date\t".$_[0]."\n";
	flock(STDERR, LOCK_UN);
}

sub HTTP_Responce(@) {
	my $client;
	my $code;
	($client, $code) = @_;
	print $client "HTTP/1.0 $code", Socket::CRLF;
	print $client Socket::CRLF;
	print $client <<DATA;
<html>
<head>
<title>$code</title>
</head>
<body>
<h1>$code</h1>
<p><br>
Please, contact the <a href="mailto:fsgteam\@ispras.ru">ISP RAS support team</a> to report about this error.<br>
The complete log file should be located at the <font color="green">$SERVER_ROOT/server.log.8888</font> on the server.</p>
</body>
</html>
DATA
	LOG("[$$]: Responce: $code");
}

my $port = (shift or 8888);

LOG("Starting httpd on port $port");
my $server = new IO::Socket::INET(Proto => 'tcp',
				LocalPort => $port,
				Listen => SOMAXCONN,
				Reuse => 1);
die "Could not create server socket: $!" unless $server;

if (open(EXT_INFO, $SERVER_ROOT."/mime.types")) {
	while (<EXT_INFO>) {
		if (/^[^\s#]/) {
			s/[\r\n]//g;
			my ($read_type, @read_exts) = split(/\s+/);
			foreach (@read_exts) {
				$mime_types{$_} = $read_type;
			}
		}
	}
	close(EXT_INFO);
}

while (my $client = $server->accept()) {
	$client->autoflush(1);

	my $child = fork();
	if (!defined($child)) {
		close($client);
		LOG("Connection accepted. ERROR: Could not start fork!");
		HTTP_Responce($client, "500 Internal Server Error");
	}
	elsif ($child == 0) {
		select(undef, undef, undef, 0.1);
		chdir($DOCUMENT_ROOT);
		LOG("Connection accepted. ID: $$.");
		my %request = ();
		$request{COOKIES} = "";
		{
			local $/ = Socket::CRLF;
			my $content_length = 0;

			while (<$client>) {
				chomp;
				## headers
				if (/\s*(\w+)\s*([^\s\?]+)(\?([^\s]*))?\s*HTTP\/(\d.\d)/) {
					$request{METHOD} = uc($1);
					$request{URL} = $2;
					$request{HTTP_VERSION} = $5;
					$request{GET_ARGS} = $4 ? $4 : "";
					$request{GET_ARGS} =~ s/#.*$//;
					LOG("[$$]: Requested: $_");
				}
				elsif (m/Content-length:/i) {
					(my $name, $content_length) = split(/:\s*/, $_, 2);
				}
				elsif (m/Cookie:/i) {
					(my $name, $request{COOKIES}) = split(/:\s*/, $_, 2);
				}
				elsif (/^$/) {
					if ($content_length) {
						read($client, $request{POST_ARGS}, $content_length);
					}
					else {
						$request{POST_ARGS} = "";
					}
					last;
				}
			}
		}

		## process request
		if (($request{METHOD} eq 'GET') or ($request{METHOD} eq 'POST') or ($request{METHOD} eq 'HEAD')) {
			# Forbid '..' sequence inside URL
			if ($request{URL} =~ m/\.\./) {
				HTTP_Responce($client, "403 Forbidden");
			}
			else {
				if ($request{URL} eq "/") {
					foreach(@index_docs) {
						if (-f $DOCUMENT_ROOT."/".$_) {
							$request{URL} = "/".$_;
							last;
						}
					}
				}
				my $localfile = $DOCUMENT_ROOT.$request{URL};
				if (-f $localfile)
				{
					my $buffer;
					my $is_executable = ($request{URL} =~ m/\.(pl|cgi|php)$/);
					my $opened;
					if ($is_executable) {
						$opened = open(FILE, "$localfile \"$request{GET_ARGS}\" \"$request{POST_ARGS}\" \"$request{COOKIES}\" ".$client->peerhost()." 2>/tmp/tests_error.$$ |");
					}
					else {
						$opened = open(FILE, $localfile);
					}
					if ($opened)
					{
						my $mime_type;
						my $first_line;
						my $null_output = 0;
						if ($is_executable) {
							$first_line = <FILE>;
							if (defined($first_line)) {
								print $client $first_line if ($client->connected);
								$first_line =~ s/[\r\n]//g;
								my $resp = $first_line;
								$first_line .= " <from script>";
								while($resp ne "") {
									$resp = <FILE>;
									print $client $resp if ($client->connected);
									$resp =~ s/[\r\n]//g;
								}
							}
							else {
								$null_output = 1;
								$first_line = "<null> <from script>";
							}
						}
						else {
							my $ext = ($request{URL} =~ m/^.*\.([^\.]*)$/) ? $1 : "";
							$mime_type = ($mime_types{$ext} or "text/html");
							$first_line = "HTTP/1.0 200 OK ($mime_type)";
							print $client "HTTP/1.0 200 OK", Socket::CRLF;
							print $client "Content-type: $mime_type", Socket::CRLF;
							print $client Socket::CRLF;
						}
						while(<FILE>) {
							print $client $_ if ($client->connected and ($request{METHOD} ne 'HEAD'));
						}
						close(FILE);
						if ($is_executable) {
							print STDERR `cat /tmp/tests_error.$$`;
							if ($null_output and (-s "/tmp/tests_error.$$" != 0)) {
								LOG("[$$]: Script execution error!");
								HTTP_Responce($client, "500 Internal Server Error");
							}
							else {
								LOG("[$$]: Responce: $first_line");
							}
							unlink("/tmp/tests_error.$$");
						}
						else {
							LOG("[$$]: Responce: $first_line");
						}
					}
					else {
						HTTP_Responce($client, "403 Forbidden");
					}
				}
				else {
					HTTP_Responce($client, "404 Not Found");
				}
			}
		}
		else {
			HTTP_Responce($client, "501 Not Implemented");
		}
		LOG("[$$]: Processing finished.");
		close($client);
		close($server);
		exit;
	}
	else {
		close($client);
		my $kid;
		do {
			$kid = waitpid(-1, WNOHANG);
		} while $kid > 0;
	}
}
